/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.kenai.com
 * SCM: https://kenai.com/hg/openbluesky~src
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.nodes.role.impl;

import javax.annotation.Nonnull;
import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.List;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.nodes.NodeEvent;
import org.openide.nodes.NodeListener;
import org.openide.nodes.NodeMemberEvent;
import org.openide.nodes.NodeReorderEvent;
import it.tidalwave.util.Finder;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.role.Composite;
import it.tidalwave.netbeans.nodes.NodePresentationModel;
import it.tidalwave.netbeans.nodes.GenericFilterNode;

/*********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class RearrangingChildren extends Children.Keys<Object> // FIXME: SafeChildFactory?
  {
    @Nonnull
    private final Composite<?, ?> composite;
    
    private final Children delegateChildren;
    
    private final ReflectiveNodeFactory nodeFactory;

    public RearrangingChildren (final @Nonnull Node delegateNode,
                                final @Nonnull Composite<?, ?> composite,
                                final @Nonnull Object[] extraArgs) 
      {
        this.composite = composite;
        this.delegateChildren = delegateNode.getChildren();
        this.nodeFactory = new ReflectiveNodeFactory(extraArgs);
                
        delegateNode.addNodeListener(new NodeListener() 
          {
            @Override
            public void childrenAdded (NodeMemberEvent ev) 
              {
                loadNodes();
              }

            @Override
            public void childrenRemoved (NodeMemberEvent ev)  
              {
                loadNodes();
              }

            @Override
            public void childrenReordered (NodeReorderEvent ev) 
              {
                // ignored, since we're the ones setting the order
              }

            @Override
            public void nodeDestroyed (NodeEvent ev) 
              {
                // ok, ignored
              }

            @Override
            public void propertyChange (PropertyChangeEvent evt) 
              {
                // ok, ignored
              }
          });
        
        loadNodes(); // FIXME: in addNotify()
      }

    public void loadNodes() 
      {
        final List<Object> keys = new ArrayList<Object>();
        
        final Node[] nn = delegateChildren.getNodes();
        
        if ((nn.length == 1) && isWaitingNode(nn[0]))
          {
            keys.add(nn[0]);               
          }
        
        else if (nn.length > 0)
          {
            for (final Object o : composite.findChildren().results())
              {
                try 
                  {
                    keys.add(findRepresentantOf(o));
                  }
                catch (NotFoundException e)
                  {
                    keys.add(o);                     
                  }
              }
          }
        
        setKeys(keys);
      }

    @Override @Nonnull
    protected Node[] createNodes (final @Nonnull Object key) 
      {
        if (key instanceof Node)
          {
            final Node node = (Node)key;
            
            if (isWaitingNode(node))
              {
                return new Node[] { new GenericFilterNode(node) };  
              }
            
            return new Node[] { nodeFactory.createNode(getNode().getClass(), node) };
          }
        else
          {
            return new Node[] { nodeFactory.createNode(getNode().getClass(), new NodePresentationModel(key)) };
          }
      }
    
    @Nonnull
    private Node findRepresentantOf (final Object o)
      throws NotFoundException
      {
        for (final Node n : delegateChildren.getNodes())
          {
            final Object o2 = n.getLookup().lookup(o.getClass());
            
            // TODO: comparing references might be not enough. For instance, if FileModel in NodeExamples8 doesn't
            // implement equals() this will never work and cause the re-creation of many Nodes. This is because
            // FileModel always recreate new instances in findChildren(). Actually, this could be just seen as a
            // problem of FileModel, that should go lightweight. Just document this.
            // You could think of explicitly probing Id for an Identifiable, that could be faster.
            if (o2 == o)
              {
                return n;  
              }
          }
        
        throw new NotFoundException();
      }
    
    private static boolean isWaitingNode (final @Nonnull Node node) // FIXME: improve it
      {
        return node.getClass().getName().contains("Wait");
      }
  }
